#include <panel-applet.h>
#include <gconf/gconf-client.h>
#include <libgnomeui/libgnomeui.h>
#include <panel-applet-gconf.h>
#include <libgnomevfs/gnome-vfs.h>
#include <glade/glade.h>
#include <gtk/gtk.h>
#include <string.h>
#include "libgtodo.h"

GTodoClient *client;
GtkListStore *list;
GtkWidget  *window, *label;
GtkWidget *tree;
PanelApplet *applet;
void fill_todo_list();
#define UNIX_PATH_MAX 108
void show_preferences();

void start_gtodo(GtkWidget *wid, GTime id);


void start_gtodo_no_id() { start_gtodo(NULL, 0); }

static const BonoboUIVerb gtodo_applet_verbs [] = {
	BONOBO_UI_UNSAFE_VERB ("Preferences", show_preferences),
	BONOBO_UI_UNSAFE_VERB ("Start Gtodo", start_gtodo_no_id),
	BONOBO_UI_VERB_END, 
};


void start_gtodo(GtkWidget *wid, GTime id)
{
	gchar *command;
	if(id == -1) command = g_strdup("gtodo");
	else command = g_strdup_printf("gtodo -s %i", (gint)id);
	g_spawn_command_line_async(command, NULL);
	g_free(command);    
}


void due_color_changed(GnomeColorPicker *cp)
{
	gushort r, g,b,a;
	gnome_color_picker_get_i16(cp, &r, &g, &b,&a);
	panel_applet_gconf_set_int(PANEL_APPLET(applet), "due_red",(gint)r, NULL);
	panel_applet_gconf_set_int(PANEL_APPLET(applet), "due_green",(gint)g, NULL);
	panel_applet_gconf_set_int(PANEL_APPLET(applet), "due_blue", (gint)b, NULL);
	fill_todo_list();
}

void today_color_changed(GnomeColorPicker *cp)
{
	gushort r, g,b,a;
	gnome_color_picker_get_i16(cp, &r, &g, &b,&a);
	panel_applet_gconf_set_int(PANEL_APPLET(applet), "today_red",(gint)r, NULL);
	panel_applet_gconf_set_int(PANEL_APPLET(applet), "today_green",(gint)g, NULL);
	panel_applet_gconf_set_int(PANEL_APPLET(applet), "today_blue", (gint)b, NULL);
	fill_todo_list();
}

void in_days_color_changed(GnomeColorPicker *cp)
{
	gushort r, g,b,a;
	gnome_color_picker_get_i16(cp, &r, &g, &b,&a);
	panel_applet_gconf_set_int(PANEL_APPLET(applet), "in_days_red",(gint)r, NULL);
	panel_applet_gconf_set_int(PANEL_APPLET(applet), "in_days_green",(gint)g, NULL);
	panel_applet_gconf_set_int(PANEL_APPLET(applet), "in_days_blue", (gint)b, NULL);
	fill_todo_list();
}
void show_due_changed(GtkToggleButton *tb)
{
	panel_applet_gconf_set_bool(PANEL_APPLET(applet), "show_due", 
			gtk_toggle_button_get_active(tb), NULL);
	fill_todo_list();			
}
void show_today_changed(GtkToggleButton *tb)
{
	panel_applet_gconf_set_bool(PANEL_APPLET(applet), "show_today", 
			gtk_toggle_button_get_active(tb), NULL);
	fill_todo_list();
}
void show_in_days_changed(GtkToggleButton *tb)
{
	panel_applet_gconf_set_bool(PANEL_APPLET(applet), "show_in_days", 
			gtk_toggle_button_get_active(tb), NULL);
	fill_todo_list();			
}
void show_other_changed(GtkToggleButton *tb)
{
	panel_applet_gconf_set_bool(PANEL_APPLET(applet), "show_other", 
			gtk_toggle_button_get_active(tb), NULL);
	fill_todo_list();			
}		

void show_preferences()
{
	GladeXML *xml =  glade_xml_new(GLADE_PATH"gtodo-applet.glade", "pref_dialog", NULL);
	gnome_color_picker_set_i16((GnomeColorPicker *)glade_xml_get_widget(xml, "cp_due"), 
			panel_applet_gconf_get_int(PANEL_APPLET(applet), "due_red", NULL),
			panel_applet_gconf_get_int(PANEL_APPLET(applet), "due_green", NULL),	
			panel_applet_gconf_get_int(PANEL_APPLET(applet), "due_blue", NULL),	
			0);
	gnome_color_picker_set_i16((GnomeColorPicker *)glade_xml_get_widget(xml, "cp_today"),
			panel_applet_gconf_get_int(PANEL_APPLET(applet), "today_red", NULL),
			panel_applet_gconf_get_int(PANEL_APPLET(applet), "today_green", NULL),	
			panel_applet_gconf_get_int(PANEL_APPLET(applet), "today_blue", NULL),	
			0);
	gnome_color_picker_set_i16((GnomeColorPicker *)glade_xml_get_widget(xml, "cp_in_days"), 
			panel_applet_gconf_get_int(PANEL_APPLET(applet), "in_days_red", NULL),
			panel_applet_gconf_get_int(PANEL_APPLET(applet), "in_days_green", NULL),	
			panel_applet_gconf_get_int(PANEL_APPLET(applet), "in_days_blue", NULL),	
			0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "cb_due")), 
			panel_applet_gconf_get_bool(PANEL_APPLET(applet), "show_due", NULL));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "cb_today")), 
			panel_applet_gconf_get_bool(PANEL_APPLET(applet), "show_today", NULL));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "cb_in_days")), 
			panel_applet_gconf_get_bool(PANEL_APPLET(applet), "show_in_days", NULL));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "cb_other")), 
			panel_applet_gconf_get_bool(PANEL_APPLET(applet), "show_other", NULL));

	/* connect the signals in the interface */
	glade_xml_signal_autoconnect(xml);

	gtk_widget_show_all(GTK_WIDGET(glade_xml_get_widget(xml, "pref_dialog")));
	gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "pref_dialog")));
	gtk_widget_destroy(glade_xml_get_widget(xml, "pref_dialog"));
	g_object_unref(xml);
}

/* move the applet to the correct location when opening it. */
void applet_move(GtkWidget *wid, GtkAllocation *a)
{
	gint x=0, y=0, width=0, height=0;
	gint wwidth, wheight;
	PanelAppletOrient ori = panel_applet_get_orient(applet);
	gtk_window_get_size (GTK_WINDOW(window), &wwidth, &wheight);
	gdk_window_get_origin (wid->window, &x, &y);
	gdk_drawable_get_size (wid->window, &width, &height);
//	width = 0, height = 0;

	switch (ori) {
		case PANEL_APPLET_ORIENT_DOWN:
			x += (width - wwidth) / 2;
			y += height;
			break;

		case PANEL_APPLET_ORIENT_UP:
			x += (width - wwidth) / 2;
			y -= wheight;
			break;

		case PANEL_APPLET_ORIENT_LEFT:
			x -= wwidth;
			y += (height - wheight) / 2;			
			break;

		case PANEL_APPLET_ORIENT_RIGHT:
			x += width;
			y += (height - wheight) / 2;			
			break;

		default:
			break;
	}

	if (x < 0) {
		x = 0;
	}
	if (y < 0) {
		y = 0;
	}
	gtk_window_move (GTK_WINDOW (window), x, y);
}

/* load the image size it to the panel size with a 5 px border and draw it into the gtkimage */
/*void image_resize()
{
	GdkPixbuf *temp;
	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file(DATADIR"/pixmaps/gtodo.png", NULL);
	gint width = panel_applet_get_size(applet) -10;
	temp = gdk_pixbuf_scale_simple(pixbuf, width,width, GDK_INTERP_NEAREST);
	gtk_image_set_from_pixbuf(GTK_IMAGE(image), temp);
	g_object_unref(temp);
	g_object_unref(pixbuf);
}
*/
/* quick hack to make the right mouse menu work  */
gboolean button_press(GtkWidget *widget, GdkEventButton *event,PanelApplet *applet)
{
	if(event->button > 1)
	{
		gtk_propagate_event((GtkWidget *)applet, (GdkEvent *) event);
	}
	else
	{
	}
	return FALSE;
}

/* show hide the todo list window */
/* make sure when showing it's at the right location */
void button_toggled(GtkToggleButton *tb)
{
	if(gtk_toggle_button_get_active(tb))
	{
		applet_move(applet, NULL);
		gtk_widget_show_all(window);
	}
	else{
		gtk_widget_hide_all(window);
	}    
}

void mark_completed(GtkWidget *wid, guint64 id)
{
	GTodoItem *item = gtodo_client_get_todo_item_from_id(client, id);
	gtodo_todo_item_set_done(item, TRUE);
	gtodo_client_edit_todo_item(client, item);
}

void mark_completed_row(GtkCellRenderer *renderer,const char *path)
{
	GtkTreeIter iter;
	GtkTreeModel *model = GTK_TREE_MODEL(list);
	guint64 id=0;
	if(gtk_tree_model_get_iter_from_string(model, &iter, path))
	{
		gtk_tree_model_get(model, &iter, 0, &id, -1);
		mark_completed(NULL, id);
	}
}




/* create context menu */

int create_context_menu(GtkWidget *widget, GdkEventButton *event)
{
	GtkWidget *menu;
	GtkWidget *menuitem;
	GtkTreeSelection * selection;
	GtkTreeIter iter;
	GtkTreeModel *model = GTK_TREE_MODEL(list);
	guint64 id=0;
	if(event->button != 3) return FALSE;
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));	
	if(!gtk_tree_selection_get_selected(selection, &model, &iter))
	{
		return FALSE;
	}
	gtk_tree_model_get(model, &iter, 0, &id, -1);
	g_print("id %i\n", (gint)id);
	menu = gtk_menu_new();

	/* the mark completed menu item */
	menuitem = gtk_menu_item_new_with_label("Mark as completed");
	g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(mark_completed), id);
	gtk_menu_append(menu, menuitem);

	menuitem = gtk_menu_item_new_with_label("Open in gtodo");
	g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(start_gtodo), id);
	gtk_menu_append(menu, menuitem);

	gtk_widget_show_all(menu);
	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time()); 

}

/* create the todo list window */
void create_popdown_window()
{

	GtkWidget *sw,*vbox;
	GtkTreeViewColumn  *column = NULL;
	GtkCellRenderer *renderer;
	GtkWidget *frame, *frame2;
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	vbox = gtk_vbox_new(FALSE,6);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
	list = gtk_list_store_new(6, GTK_TYPE_INT, GTK_TYPE_STRING,GTK_TYPE_STRING, GTK_TYPE_STRING, G_TYPE_BOOLEAN, GDK_TYPE_COLOR);
	tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(list));

	/* outer frame, to create some sort of border */
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
	gtk_container_add(GTK_CONTAINER(window),frame);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	/* inerframe */
	sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	frame2 = gtk_frame_new("tasks");
	gtk_label_set_markup(GTK_LABEL(gtk_frame_get_label_widget(GTK_FRAME(frame2))), "<b>tasks</b>");
	gtk_frame_set_shadow_type (GTK_FRAME (frame2), GTK_SHADOW_NONE);
	gtk_box_pack_start(GTK_BOX(vbox),frame2, TRUE, TRUE,0);
	gtk_container_add(GTK_CONTAINER(frame2), sw);
	gtk_container_add(GTK_CONTAINER(sw), tree);    


	gtk_tree_view_column_set_sort_column_id(column, 2);

	renderer = gtk_cell_renderer_toggle_new();
	column = gtk_tree_view_column_new_with_attributes(_("Summary"), renderer,"foreground-set", 4,"foreground-gdk", 5, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);    
	g_signal_connect(G_OBJECT(renderer), "toggled", G_CALLBACK(mark_completed_row), NULL);
	
	renderer =  gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(_("Summary"), renderer, "text" , 2,"foreground-set", 4,"foreground-gdk", 5, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);    
	gtk_tree_view_column_set_sort_column_id(column, 3);
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree), FALSE);


	gtk_window_resize(window, 200, 150);
	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(window), TRUE);
	gtk_window_set_decorated(GTK_WINDOW(window), FALSE);

	g_signal_connect(G_OBJECT(tree), "button-press-event", G_CALLBACK(create_context_menu), NULL);
}

/* Fill the gtodo list */
void fill_todo_list()
{
	GTodoList *todolist;
	GdkColor due, today, in_days;
	int due_counter = 0;
	gtk_list_store_clear(list);
	todolist = gtodo_client_get_todo_item_list(client, NULL);
	if(todolist == NULL) return;

	/* should be a better way for this, but the colorpicker + gconf is limited */
	due.red = panel_applet_gconf_get_int(PANEL_APPLET(applet), "due_red", NULL);
	due.green = panel_applet_gconf_get_int(PANEL_APPLET(applet), "due_green", NULL);
	due.blue = panel_applet_gconf_get_int(PANEL_APPLET(applet), "due_blue", NULL);

	today.red = panel_applet_gconf_get_int(PANEL_APPLET(applet), "today_red", NULL);
	today.green = panel_applet_gconf_get_int(PANEL_APPLET(applet), "today_green", NULL);
	today.blue = panel_applet_gconf_get_int(PANEL_APPLET(applet), "today_blue", NULL);

	in_days.red = panel_applet_gconf_get_int(PANEL_APPLET(applet), "in_days_red", NULL);
	in_days.green = panel_applet_gconf_get_int(PANEL_APPLET(applet), "in_days_green", NULL);
	in_days.blue = panel_applet_gconf_get_int(PANEL_APPLET(applet), "in_days_blue", NULL);
	do{
		GtkTreeIter iter;
		char *buf;
		GTodoItem *item = gtodo_client_get_todo_item_from_list(todolist);
		int diff = gtodo_todo_item_check_due(item);
		if(!gtodo_todo_item_get_done(item))
			if((diff > 0 && diff != GTODO_NO_DUE_DATE && panel_applet_gconf_get_bool(PANEL_APPLET(applet), "show_due", NULL))
					|| (diff == 0 && panel_applet_gconf_get_bool(PANEL_APPLET(applet), "show_today", NULL))
					|| panel_applet_gconf_get_bool(PANEL_APPLET(applet), "show_other", NULL)
					|| (diff > -7 && diff <0 && panel_applet_gconf_get_bool(PANEL_APPLET(applet), "show_in_days", NULL)))
			{
				if(diff  >= 0 && diff != GTODO_NO_DUE_DATE)
				{
					due_counter++;
				}
				if(gtodo_todo_item_get_priority(item) == GTODO_PRIORITY_LOW)
				{
					buf = g_strdup("Low");
				}
				else if (gtodo_todo_item_get_priority(item) == GTODO_PRIORITY_MEDIUM)
				{
					buf = g_strdup("Medium");			
				}
				else buf = g_strdup("High");			
				gtk_list_store_append(list, &iter);
				gtk_list_store_set(list, &iter, 4, FALSE, -1);
				if(diff > 0&& diff != GTODO_NO_DUE_DATE && panel_applet_gconf_get_bool(PANEL_APPLET(applet), "show_due", NULL) ) 
				{
					gtk_list_store_set(list, &iter, 4, TRUE,5, &due, -1);
				}
				else if (diff == 0 && panel_applet_gconf_get_bool(PANEL_APPLET(applet), "show_today", NULL))
				{
					gtk_list_store_set(list, &iter, 4, TRUE, 5, &today, -1);
				}
				else if (diff < 0 && diff > -7 && panel_applet_gconf_get_bool(PANEL_APPLET(applet), "show_in_days", NULL)) 
				{
					gtk_list_store_set(list, &iter, 4, TRUE, 5, &in_days, -1);
				}
				gtk_list_store_set(list, &iter, 
						0, gtodo_todo_item_get_id(item), 
						1, buf,
						2, gtodo_todo_item_get_summary(item),
						-1);
				g_free(buf);
			}			

	}while(gtodo_client_get_list_next(todolist));
	gtodo_client_free_todo_item_list(client, todolist);

	if(due_counter > 1)
	{
		gchar *string = g_strdup_printf("%s: %i", _("tasks due"),  due_counter);
		gtk_label_set_text(label, string);                          		
		g_free(string);
	}
	else
	{
		gchar *string = g_strdup_printf("%s: %i", _("task due"),  due_counter);
		gtk_label_set_text(label, string);
		g_free(string);

	}
}

static gboolean gtodo_applet_fill (PanelApplet *ap,   const gchar *iid,	gpointer     data)
{
	GtkWidget *button;
	applet = ap;
       	/* get a global pointer to the applet  */
	/* check if this is the good applet I guess, I just copied the stuff */
	if (strcmp (iid, "OAFIID:gtodo-applet") != 0)
	{
		return FALSE;
	}


	/* the button the user will see on the bar */
	button = gtk_toggle_button_new();
	/* create an empty image that is placed on the button */
	label = gtk_label_new("tasks due: 0");
	/* now load and draw the image in the right size onto the toggle button */
//	image_resize();
	/* function that resizes the image when the panel size is changed */
/*	g_signal_connect (applet,
			"change_size",
			G_CALLBACK (image_resize),
			NULL);
*/
	/* add the image to the button */
	gtk_container_add (GTK_CONTAINER (button), label);

	/* create the "popdown" window that appears when the button is clicked */
	create_popdown_window();

	/* add the button to the applet and connect signals to handle a click */
	gtk_container_add (GTK_CONTAINER (applet), button);
	g_signal_connect(G_OBJECT(button), "button-press-event", G_CALLBACK(button_press), applet);
	g_signal_connect(G_OBJECT(button), "toggled", G_CALLBACK(button_toggled), NULL);	

	/* show the applet */
	gtk_widget_show_all (GTK_WIDGET (applet));

	/* start the gtodo backend */
	client = gtodo_client_new_default(NULL);
	if(client == NULL)
	{
		/* I should **never** end up here */
		/* I should make a nice error box here shouldn't I?*/
		return FALSE;
	}

	panel_applet_add_preferences (applet, "/schemas/apps/gtodo_applet/prefs", NULL);

	/* fill the list with todo items */
	fill_todo_list();
	/* connect a signal that reload the signal when something changes */
	gtodo_client_set_changed_callback(client,(void *)fill_todo_list, NULL);


	panel_applet_setup_menu_from_file (applet,
			NULL,
			"gtodo-applet.xml",
			NULL, 
			gtodo_applet_verbs,
			NULL);
	return TRUE;
}


PANEL_APPLET_BONOBO_FACTORY ("OAFIID:gtodo_applet_Factory",PANEL_TYPE_APPLET,"Gtodo Applet","0",gtodo_applet_fill,NULL);

